/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMControl.h
//  aria
//
//  Created by Yohan Lasorsa on 01/07/09.
//  Copyright 2009 INRIA, Team WAM. All rights reserved.
//

#import "AMControllable.h"
#import <Foundation/Foundation.h>


/* Constants of control names */
static const NSString *const AM_TEMPO_CONTROL = @"TempoControl";
static const NSString *const AM_TRIGGER_CONTROL = @"TriggerControl";
static const NSString *const AM_SPAWN_CONTROL = @"SpawnControl";
static const NSString *const AM_MIX_2D_CONTROL = @"Mix2DControl";
static const NSString *const AM_MIX_3D_CONTROL = @"Mix3DControl";
static const NSString *const AM_SOUND_CONTROL = @"SoundControl";

/**
 Superclass of all control objects.
 This class is not destined to be used directly, but to be derived.
 */
@interface AMControl : NSObject {
	// Control attributes
	id parent;
}

/**
 The parent object holding the control.
 */
@property (nonatomic, assign) id parent;

/**
 Creates a control object with the given name.
 If the name does not bind to a valid control, nil is returned.
 @param name The name of the control to create.
 @param controllable The object parameterized by the control.
 @return The corresponding control object if it can be created or nil else.
 */
+ (AMControl *)controlWithName:(NSString *)_name controllable:(id)_controllable;

/**
 Sets the value of the specified attribute.
 (THIS METHOD SHOULD ALWAYS BE OVERRIDEN IN SUBCLASSES.)
 @param value The value to set.
 @param attribute The attribute to set, as specified in the control attributes enumeration.
 */
- (void)setValue:(id)_value forAttribute:(int)_attribute;

/**
 Gets the value of the specified attribute.
 (THIS METHOD SHOULD ALWAYS BE OVERRIDEN IN SUBCLASSES.)
 @param attribute The attribute to get the value of.
 @return The value of the attribute, or nil if no attribute with this name was found.
 */
- (id)valueOfAttribute:(int)_attribute;

/**
 Gets the attribute code (enum int value) for the specified attribute name.
 (THIS METHOD SHOULD ALWAYS BE OVERRIDEN IN SUBCLASSES.)
 @param atribute The name of the attribute.
 @return The enumeration integer value of the attribute, or -1 if the attribute doesn't exists for the control.
 */
- (int)attributeCode:(NSString *)_attribute;

@end
