/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMControl.mm
//  aria
//
//  Created by Yohan Lasorsa on 01/07/09.
//  Copyright 2009 INRIA, Team WAM. All rights reserved.
//

#import "AMControl.h"
#import "AMMix2DControl.h"
#import "AMMix3DControl.h"
#import "AMSoundControl.h"
#import "AMTempoControl.h"
#import "AMTriggerControl.h"
#import "AMSpawnControl.h"


@implementation AMControl

@synthesize parent;

- (id)init
{
	if (self = [super init]) {
		parent = nil;
	}
	return self;
}

+ (AMControl *)controlWithName:(NSString *)_name controllable:(id)_controllable
{
	AMControl *control = nil;
	NSSet *controls = [(id<AMControllable>)_controllable getControls];
	
	// check if the requested control is available for the given object
	if ([controls containsObject:_name]) {
		
		// create the requested control
		if ([_name isEqualToString:AM_MIX_2D_CONTROL])
			control = [[AMMix2DControl alloc] init];
		else if ([_name isEqualToString:AM_MIX_3D_CONTROL])
			control = [[AMMix3DControl alloc] init];			
		else if ([_name isEqualToString:AM_SOUND_CONTROL])
			control = [[AMSoundControl alloc] init];
		else if ([_name isEqualToString:AM_TEMPO_CONTROL])
			control = [[AMTempoControl alloc] init];
		else if ([_name isEqualToString:AM_TRIGGER_CONTROL])
			control = [[AMTriggerControl alloc] init];
		else if ([_name isEqualToString:AM_SPAWN_CONTROL])
			control = [[AMSpawnControl alloc] init];
		
		// TODO: complete with other controls
		
		// set ownership of the control
		if (control != nil) {
			control.parent = _controllable;
			[control autorelease];
		}
	}
	
	return control;
}

- (void)setValue:(id)_value forAttribute:(int)_attribute {
	// needs to be overriden in subclasses
}

- (id)valueOfAttribute:(int)_attribute {
	// needs to be overriden in subclasses
	return nil;
}

- (int)attributeCode:(NSString *)_attribute {
	// needs to be overriden in subclasses
	return -1;
}

@end
