/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMMix2DControl.h
//  aria
//
//  Created by Yohan on 04/05/10.
//  Copyright 2010 INRIA, Team WAM. All rights reserved.
//

#import "AMControl.h"
#import "fmod.hpp"
#import "fmod_errors.h"
#import <Foundation/Foundation.h>


/* Constants of attributes names */
static const NSString *const AM_MIX2D_CONTROL_ATTRIBUTE_VOLUME = @"volume";
static const NSString *const AM_MIX2D_CONTROL_ATTRIBUTE_MUTE = @"mute";
static const NSString *const AM_MIX2D_CONTROL_ATTRIBUTE_PAN = @"pan";

/** Attributes of the control. */
enum AMMix2DControlAttributes {
	AM_M2D_VOLUME = 0,
	AM_M2D_MUTE,
	AM_M2D_PAN
};


/**
 Allows to control the 2D mix parameters, such as volume and panorama.
 */
@interface AMMix2DControl : AMControl <NSCopying> {
	// Control attributes
	int volume;
	BOOL mute;
	int pan;
}

/**
 The output volume of the audio track.
 Value must be between 0 and 100. Default value is 100;
 */
@property (nonatomic) int volume;

/**
 The mute status of the audio tracks.
 Default value is NO.
 */
@property (nonatomic) BOOL mute;

/**
 The panning of the audio track.
 The value must be between -100 and 100, 0 being the middle, -100 full left and 100 full right.
 Default value is 0.
 */
@property (nonatomic) int pan;


/**
 Apply the 2D parameters to the given audio object instance.
 @param instance A CueInstance or SoundInstance object on which you want to apply these 2D parameters.
 */
- (void)applySettingsToInstance:(id)_instance;

@end
