/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMMix2DControl.mm
//  aria
//
//  Created by Yohan on 04/05/10.
//  Copyright 2010 INRIA, Team WAM. All rights reserved.
//

#import "AMMix2DControl.h"
#import "AMSection.h"
#import "AMSoundInstance.h"


@implementation AMMix2DControl

@synthesize volume;
@synthesize mute;
@synthesize pan;

- (id)init
{
	if (self = [super init]) {
		volume = 100;
		mute = NO;
		pan = 0;
	}
	return self;
}

- (id)initWithControl:(AMMix2DControl *)_control
{
	if (self = [super init]) {
		volume = _control.volume;
		mute = _control.mute;
		pan = _control.pan;
	}
	return self;
}

- (id)copyWithZone:(NSZone *)_zone
{
	return [[[self class] allocWithZone:_zone] initWithControl:self];
}
	
- (void)setVolume:(int)_volume
{
	volume = _volume;
	
	// apply volume
	Class parentClass = [parent class];
	if (parentClass == [AMSection class]) {
		[(AMSection *)parent getFMODChannelGroup]->setVolume(((double)volume)/100.0);
	} else if (parentClass == [AMSoundInstance class]) {
		(*[(AMSoundInstance *)parent getFMODChannelPtr])->setVolume(((double)volume)/100.0);
	}
}

- (void)setMute:(BOOL)_mute
{
	mute = _mute;
	
	// apply mute
	Class parentClass = [parent class];
	if (parentClass == [AMSection class]) {
		[(AMSection *)parent getFMODChannelGroup]->setMute(mute);
	} else if (parentClass == [AMSoundInstance class]) {
		(*[(AMSoundInstance *)parent getFMODChannelPtr])->setMute(mute);
	}
}

- (void)setPan:(int)_pan
{
	pan = _pan;
	
	// apply pan
	Class parentClass = [parent class];
	if (parentClass == [AMSection class]) {
		[(AMSection *)parent getFMODChannelGroup]->overridePan(((double)pan)/100.0);
	} else if (parentClass == [AMSoundInstance class]) {
		(*[(AMSoundInstance *)parent getFMODChannelPtr])->setPan(((double)pan)/100.0);
	}
}

- (void)setValue:(id)_value forAttribute:(int)_attribute {
	if (_attribute == AM_M2D_VOLUME)
		self.volume = [_value floatValue];
	else if (_attribute == AM_M2D_MUTE)
		self.mute = [_value floatValue];
	else if (_attribute == AM_M2D_PAN)
		self.pan = [_value floatValue];
}

- (id)valueOfAttribute:(int)_attribute {
	if (_attribute == AM_M2D_VOLUME)
		return [NSNumber numberWithFloat:self.volume];
	else if (_attribute == AM_M2D_MUTE)
		return [NSNumber numberWithFloat:self.mute];
	else if (_attribute == AM_M2D_PAN)
		return [NSNumber numberWithFloat:self.pan];
	return nil;
}

- (int)attributeCode:(NSString *)_attribute {
	if ([AM_MIX2D_CONTROL_ATTRIBUTE_VOLUME isEqual:_attribute])
		return AM_M2D_VOLUME;
	else if ([AM_MIX2D_CONTROL_ATTRIBUTE_MUTE isEqual:_attribute])
		return AM_M2D_MUTE;
	else if ([AM_MIX2D_CONTROL_ATTRIBUTE_PAN isEqual:_attribute])
		return AM_M2D_PAN;
	return -1;
}

- (void)applySettingsToInstance:(id)_instance
{
	if ([_instance isMemberOfClass:[AMSoundInstance class]]) {
		// get FMOD channel
		FMOD::Channel *chan = *[(AMSoundInstance *)_instance getFMODChannelPtr];
		
		// set 2D parameters
		chan->setPan(((double)pan)/100.0);
		chan->setVolume(((double)volume)/100.0);
		chan->setMute(mute);
		
	} else if ([_instance isMemberOfClass:[AMCueInstance class]]) {
		// get currently active instances
		AMCueInstance *cueInstance = (AMCueInstance *)_instance;
		AMChunkInstance *chkInstance = [cueInstance getChunkInstance1];
		AMSoundInstance *sndInstance = nil;
		
		// 1st chunk instance
		if (chkInstance != nil)
			sndInstance = [chkInstance getSoundInstance];
		if (sndInstance != nil)
			[self applySettingsToInstance:sndInstance];
		
		// 2nd chunk instance
		chkInstance = [cueInstance getChunkInstance2];
		if (chkInstance != nil)
			sndInstance = [chkInstance getSoundInstance];
		if (sndInstance != nil)
			[self applySettingsToInstance:sndInstance];
	}
}

@end
