/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMMix3DControl.h
//  aria
//
//  Created by Yohan on 05/05/10.
//  Copyright 2010 INRIA, Team WAM. All rights reserved.
//

#import "AMControl.h"
#import "AMCueInstance.h"
#import "AMSoundInstance.h"
#import "a2ml.h"
#import <Foundation/Foundation.h>


/* Constants of attributes names */
static const NSString *const AM_MIX3D_CONTROL_ATTRIBUTE_POSITION = @"position";
static const NSString *const AM_MIX3D_CONTROL_ATTRIBUTE_POSITION_X = @"position.x";
static const NSString *const AM_MIX3D_CONTROL_ATTRIBUTE_POSITION_Y = @"position.y";
static const NSString *const AM_MIX3D_CONTROL_ATTRIBUTE_POSITION_Z = @"position.z";
static const NSString *const AM_MIX3D_CONTROL_ATTRIBUTE_AUTODOPPLER = @"autoDoppler";
static const NSString *const AM_MIX3D_CONTROL_ATTRIBUTE_AUTODOPPLER_FACTOR = @"autoDopplerFactor";
static const NSString *const AM_MIX3D_CONTROL_ATTRIBUTE_ORIENTATION = @"orientation";
static const NSString *const AM_MIX3D_CONTROL_ATTRIBUTE_ORIENTATION_HEADING = @"orientation.heading";
static const NSString *const AM_MIX3D_CONTROL_ATTRIBUTE_ORIENTATION_PITCH = @"orientation.pitch";
static const NSString *const AM_MIX3D_CONTROL_ATTRIBUTE_ORIENTATION_ROLL = @"orientation.roll";
static const NSString *const AM_MIX3D_CONTROL_ATTRIBUTE_VELOCITY = @"velocity";
static const NSString *const AM_MIX3D_CONTROL_ATTRIBUTE_VELOCITY_X = @"velocity.x";
static const NSString *const AM_MIX3D_CONTROL_ATTRIBUTE_VELOCITY_Y = @"velocity.y";
static const NSString *const AM_MIX3D_CONTROL_ATTRIBUTE_VELOCITY_Z = @"velocity.z";
static const NSString *const AM_MIX3D_CONTROL_ATTRIBUTE_DOPPLER_ENABLED = @"dopplerEnabled";
static const NSString *const AM_MIX3D_CONTROL_ATTRIBUTE_DIRECTIVITY = @"directivity";
static const NSString *const AM_MIX3D_CONTROL_ATTRIBUTE_DIRECTIVITY_MIN_ANGLE = @"directivity.minAngle";
static const NSString *const AM_MIX3D_CONTROL_ATTRIBUTE_DIRECTIVITY_MAX_ANGLE = @"directivity.maxAngle";
static const NSString *const AM_MIX3D_CONTROL_ATTRIBUTE_DIRECTIVITY_REAR_LEVEL = @"directivity.rearLevel";
static const NSString *const AM_MIX3D_CONTROL_ATTRIBUTE_OCCLUSION_HF_LEVEL = @"occlusionHFLevel";
static const NSString *const AM_MIX3D_CONTROL_ATTRIBUTE_OCCLUSION_LEVEL = @"occlusionLevel";
static const NSString *const AM_MIX3D_CONTROL_ATTRIBUTE_ROLLOFF_MIN_DIST = @"rolloffMinDist";
static const NSString *const AM_MIX3D_CONTROL_ATTRIBUTE_ROLLOFF_MAX_DIST = @"rolloffMaxDist";
static const NSString *const AM_MIX3D_CONTROL_ATTRIBUTE_ROLLOFF_FACTOR = @"rolloffFactor";
static const NSString *const AM_MIX3D_CONTROL_ATTRIBUTE_MUTE_AFTER_MAX = @"muteAfterMax";

static const NSArray *const AM_MIX3D_CONTROL_ATTRIBUTES = [[NSArray alloc] initWithObjects:
														   AM_MIX3D_CONTROL_ATTRIBUTE_POSITION,
														   AM_MIX3D_CONTROL_ATTRIBUTE_POSITION_X,
														   AM_MIX3D_CONTROL_ATTRIBUTE_POSITION_Y,
														   AM_MIX3D_CONTROL_ATTRIBUTE_POSITION_Z,
														   AM_MIX3D_CONTROL_ATTRIBUTE_AUTODOPPLER,
														   AM_MIX3D_CONTROL_ATTRIBUTE_AUTODOPPLER_FACTOR,
														   AM_MIX3D_CONTROL_ATTRIBUTE_ORIENTATION,
														   AM_MIX3D_CONTROL_ATTRIBUTE_ORIENTATION_HEADING,
														   AM_MIX3D_CONTROL_ATTRIBUTE_ORIENTATION_PITCH,
														   AM_MIX3D_CONTROL_ATTRIBUTE_ORIENTATION_ROLL,
														   AM_MIX3D_CONTROL_ATTRIBUTE_VELOCITY,
														   AM_MIX3D_CONTROL_ATTRIBUTE_VELOCITY_X,
														   AM_MIX3D_CONTROL_ATTRIBUTE_VELOCITY_Y,
														   AM_MIX3D_CONTROL_ATTRIBUTE_VELOCITY_Z,
														   AM_MIX3D_CONTROL_ATTRIBUTE_DOPPLER_ENABLED,
														   AM_MIX3D_CONTROL_ATTRIBUTE_DIRECTIVITY,
														   AM_MIX3D_CONTROL_ATTRIBUTE_DIRECTIVITY_MIN_ANGLE,
														   AM_MIX3D_CONTROL_ATTRIBUTE_DIRECTIVITY_MAX_ANGLE,
														   AM_MIX3D_CONTROL_ATTRIBUTE_DIRECTIVITY_REAR_LEVEL,
														   AM_MIX3D_CONTROL_ATTRIBUTE_OCCLUSION_HF_LEVEL,
														   AM_MIX3D_CONTROL_ATTRIBUTE_OCCLUSION_LEVEL,
														   AM_MIX3D_CONTROL_ATTRIBUTE_ROLLOFF_MIN_DIST,
														   AM_MIX3D_CONTROL_ATTRIBUTE_ROLLOFF_MAX_DIST,
														   AM_MIX3D_CONTROL_ATTRIBUTE_ROLLOFF_FACTOR,
														   AM_MIX3D_CONTROL_ATTRIBUTE_MUTE_AFTER_MAX,
														   nil];

/** Attributes of the control. */
enum AMMix3DControlAttributes {
	AM_M3D_POSITION = 0,
	AM_M3D_POSITION_X,
	AM_M3D_POSITION_Y,
	AM_M3D_POSITION_Z,
	AM_M3D_AUTODOPPLER,
	AM_M3D_AUTODOPPLER_FACTOR,
	AM_M3D_ORIENTATION,
	AM_M3D_ORIENTATION_HEADING,
	AM_M3D_ORIENTATION_PITCH,
	AM_M3D_ORIENTATION_ROLL,
	AM_M3D_VELOCITY,
	AM_M3D_VELOCITY_X,
	AM_M3D_VELOCITY_Y,
	AM_M3D_VELOCITY_Z,
	AM_M3D_DOPPLER_ENABLED,
	AM_M3D_DIRECTIVITY,
	AM_M3D_DIRECTIVITY_MIN_ANGLE,
	AM_M3D_DIRECTIVITY_MAX_ANGLE,
	AM_M3D_DIRECTIVITY_REAR_LEVEL,
	AM_M3D_OCCLUSION_HF_LEVEL,
	AM_M3D_OCCLUSION_LEVEL,
	AM_M3D_ROLLOFF_MIN_DIST,
	AM_M3D_ROLLOFF_MAX_DIST,
	AM_M3D_ROLLOFF_FACTOR,
	AM_M3D_MUTE_AFTER_MAX
};


/**
 Allows to control the 3D mix parameters.
 */
@interface AMMix3DControl : AMControl <NSCopying> {
	// Control attributes
	AMVec3 position;		/* A2ML LocationControl */
	BOOL autoDoppler;		/* A2ML LocationControl */
	int autoDopplerFactor;	/* A2ML LocationControl */
	AMVec3 orientation;		/* A2ML OrientationControl */
	AMVec3 velocity;		/* A2ML DopplerControl */
	BOOL dopplerEnabled;	/* A2ML DopplerControl */
	AMVec3 directivity;		/* A2ML DirectivityControl */
	int occlusionHFLevel;	/* A2ML ObstructionControl */
	int occlusionLevel;		/* A2ML ObstructionControl */
	int rolloffMinDist;		/* A2ML DistanceAttenuationCOntrol */
	int rolloffMaxDist;		/* A2ML DistanceAttenuationCOntrol */
	int rolloffFactor;		/* A2ML DistanceAttenuationCOntrol */
	BOOL muteAfterMax;		/* A2ML DistanceAttenuationCOntrol */
	
	// No support for A2ML MacroscopicControl for now.
}

/**
 The 3D location of the sound source.
 */
@property (nonatomic) AMVec3 position;

/**
 Automatically calculate the velocity based on the location changes of the sound source if enabled.
 */
@property (nonatomic) BOOL autoDoppler;

/**
 The factor used when automatic Doppler is enabled.
 Value 0 means that the velocity will always be 0.
 */
@property (nonatomic) int autoDopplerFactor;

/**
 The orientation of the sound source, set by a 3D vector.
 */
@property (nonatomic) AMVec3 orientation;

/**
 The sound source velocity, used for the Doppler effect if enabled.
 */
@property (nonatomic) AMVec3 velocity;

/**
 Enables or not the Doppler effect.
 */
@property (nonatomic) BOOL dopplerEnabled;

/**
 The directivity of the sound source.
 */
@property (nonatomic) AMVec3 directivity;

/**
 
 */
@property (nonatomic) int occlusionHFLevel;

/**
 
 */
@property (nonatomic) int occlusionLevel;

/**
 
 */
@property (nonatomic) int rolloffMinDist;

/**
 
 */
@property (nonatomic) int rolloffMaxDist;

/**
 The attenuation factor between the min. distance and the max. distance.
 */
@property (nonatomic) int rolloffFactor;

/**
 Sets if the sound source must be muted after the max. distance.
 */
@property (nonatomic) BOOL muteAfterMax;


/**
 Apply the 3D parameters to the given audio object instance.
 @param instance A CueInstance or SoundInstance object on which you want to apply these 3D parameters.
 */
- (void)applySettingsToInstance:(id)_instance;

@end
