/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMMix3DControl.mm
//  aria
//
//  Created by Yohan on 05/05/10.
//  Copyright 2010 INRIA, Team WAM. All rights reserved.
//

#import "AMMix3DControl.h"


@implementation AMMix3DControl

@synthesize position;
@synthesize autoDoppler;
@synthesize autoDopplerFactor;
@synthesize orientation;
@synthesize velocity;
@synthesize dopplerEnabled;
@synthesize directivity;
@synthesize occlusionHFLevel;
@synthesize occlusionLevel;
@synthesize rolloffMinDist;
@synthesize rolloffMaxDist;
@synthesize rolloffFactor;
@synthesize muteAfterMax;

- (id)init
{
	if (self = [super init]) {
		position.x = 0;
		position.y = 0;
		position.z = 0;
		autoDoppler = NO;
		autoDopplerFactor = 1;
		orientation.x = 0;
		orientation.y = 0;
		orientation.z = 0;
		velocity.x = 0;
		velocity.y = 0;
		velocity.z = 0;
		dopplerEnabled = NO;
		directivity.x = 0;
		directivity.y = 0;
		directivity.z = 0;
		occlusionHFLevel = -1;
		occlusionLevel = -1;
		rolloffMinDist = 0;
		rolloffMaxDist = 0;
		rolloffFactor = -1;
		muteAfterMax = 0;
	}
	return self;
}

- (id)initWithControl:(AMMix3DControl *)_control
{
	if (self = [super init]) {
		position = _control.position;
		autoDoppler = _control.autoDoppler;
		autoDopplerFactor = _control.autoDopplerFactor;
		orientation = _control.orientation;
		velocity = _control.velocity;
		dopplerEnabled = _control.dopplerEnabled;
		directivity = _control.directivity;
		occlusionHFLevel = _control.occlusionHFLevel;
		occlusionLevel = _control.occlusionLevel;
		rolloffMinDist = _control.rolloffMinDist;
		rolloffMaxDist = _control.rolloffMaxDist;
		rolloffFactor = _control.rolloffFactor;
		muteAfterMax = _control.muteAfterMax;
	}
	return self;
}

- (id)copyWithZone:(NSZone *)_zone
{
	return [[[self class] allocWithZone:_zone] initWithControl:self];
}

/*
- (void)setPosition:(AMVec3)_position
{
	position = _position;
	[self applySettingsToInstance:parent];
}

- (void)autoDoppler:(BOOL)_enabled
{
	autoDoppler = _enabled;
	[self applySettingsToInstance:parent];
}

- (void)autoDopplerFactor:(int)_factor
{
	autoDopplerFactor = _factor;
	[self applySettingsToInstance:parent];
}

- (void)orientation:(AMVec3)_orientation
{
	orientation = _orientation;
	[self applySettingsToInstance:parent];
}

- (void)velocity:(AMVec3)_velocity
{
	velocity = _velocity;
	[self applySettingsToInstance:parent];
}

- (void)dopplerEnabled:(BOOL)_enabled
{
	dopplerEnabled = _enabled;
	[self applySettingsToInstance:parent];
}

- (void)directivity:(AMVec3)_directivity
{
	directivity = _directivity;
	[self applySettingsToInstance:parent];
}

- (void)occlusionHFLevel:(int)_HFLevel
{
	occlusionHFLevel = _HFLevel;
	[self applySettingsToInstance:parent];
}

- (void)occlusionLevel:(int)_level
{
	occlusionLevel = _level;
	[self applySettingsToInstance:parent];
}

- (void)rolloffMinDist:(int)_minDist
{
	rolloffMinDist = _minDist;
	[self applySettingsToInstance:parent];
}

- (void)rolloffMaxDist:(int)_maxDist
{
	rolloffMaxDist = _maxDist;
	[self applySettingsToInstance:parent];
}

- (void)rolloffFactor:(int)_factor
{
	rolloffFactor = _factor;
	[self applySettingsToInstance:parent];
}

- (void)muteAfterMax:(BOOL)_muteAfterMax
{
	muteAfterMax = _muteAfterMax;
	[self applySettingsToInstance:parent];
}
*/

- (void)applySettingsToInstance:(id)_instance
{
	if ([_instance isKindOfClass:[AMInstance class]]) {
		
		if ([_instance isMemberOfClass:[AMSoundInstance class]]) {
			// get FMOD channel
			FMOD::Channel *chan = *[(AMSoundInstance *)_instance getFMODChannelPtr];
			
			// set 3D parameters
			chan->setMode(FMOD_3D);
			
			if (dopplerEnabled || autoDoppler)
				chan->set3DAttributes((FMOD_VECTOR *)&position, (FMOD_VECTOR *)&velocity);
			else
				chan->set3DAttributes((FMOD_VECTOR *)&position, NULL);
			
			if (orientation.x != 0 || orientation.y != 0 || orientation.z != 0)
				chan->set3DConeOrientation((FMOD_VECTOR *)&orientation);			
			
			
			
			
			
			
			
			// TODO: complete
			
			
			
			
			
		} else if ([_instance isMemberOfClass:[AMCueInstance class]]) {
			// get currently active instances
			AMCueInstance *cueInstance = (AMCueInstance *)_instance;
			AMChunkInstance *chkInstance = [cueInstance getChunkInstance1];
			AMSoundInstance *sndInstance = nil;
			
			// 1st chunk instance
			if (chkInstance != nil)
				sndInstance = [chkInstance getSoundInstance];
			if (sndInstance != nil)
				[self applySettingsToInstance:sndInstance];
			
			// 2nd chunk instance
			chkInstance = [cueInstance getChunkInstance2];
			if (chkInstance != nil)
				sndInstance = [chkInstance getSoundInstance];
			if (sndInstance != nil)
				[self applySettingsToInstance:sndInstance];
		}
	}
}

@end
