/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMSoundControl.h
//  aria
//
//  Created by Yohan on 06/05/10.
//  Copyright 2010 INRIA, Team WAM. All rights reserved.
//

#import "AMControl.h"
#import "AMSoundInstance.h"
#import "fmod.hpp"
#import "fmod_errors.h"
#import <Foundation/Foundation.h>


/* Constants of attributes names */
static const NSString *const AM_SOUND_CONTROL_ATTRIBUTE_PITCH = @"pitch";
static const NSString *const AM_SOUND_CONTROL_ATTRIBUTE_RATE = @"rate";

/** Attributes of the control. */
enum AMSoundControlAttributes {
	AM_SND_PITCH = 0,
	AM_SND_RATE
};


/**
 Allows to control various sound parameters, such as pitch or rate.
 */
@interface AMSoundControl : AMControl <NSCopying> {
	// Control attributes
	int pitch;
	int rate;
	
	// Private vars
	float baseFrequency;
}

/**
 Sets the relative pitch shift, in milli-semitones.
 The default value is 0.
 */
@property (nonatomic) int pitch;

/**
 Sets the rate as a milli-percentage, 100'000 being the default rate.
 */
@property (nonatomic) int rate;


/**
 Apply the sound parameters to the given soun instance.
 @param instance A SoundInstance object on which you want to apply these sound parameters.
 */
- (void)applySettingsToInstance:(id)_instance;

@end
