/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMSoundControl.mm
//  aria
//
//  Created by Yohan on 06/05/10.
//  Copyright 2010 INRIA, Team WAM. All rights reserved.
//

#import "AMSoundControl.h"


@implementation AMSoundControl

@synthesize pitch;
@synthesize rate;

- (id)init
{
	if (self = [super init]) {
		pitch = 0;
		rate = 100000;
		baseFrequency = 0;
	}
	return self;
}

- (id)initWithControl:(AMSoundControl *)_control
{
	if (self = [super init]) {
		pitch = _control.pitch;
		rate = _control.rate;
		baseFrequency = 0;
	}
	return self;
}

- (id)copyWithZone:(NSZone *)_zone
{
	return [[[self class] allocWithZone:_zone] initWithControl:self];
}

- (void)setPitch:(int)_pitch
{
	pitch = _pitch;
	
	// apply pitch
	if ([parent class] == [AMSoundInstance class]) {
		FMOD::Channel *chan = *[(AMSoundInstance *)parent getFMODChannelPtr];
		
		// init base frequency if not already done
		if (baseFrequency == 0)
			chan->getFrequency(&baseFrequency);
		
		// 12 notes in an octave freq*2 = +1 octave
		chan->setFrequency((baseFrequency + (baseFrequency * (float)pitch / 12.0)) * (float)rate / 100000.0);
	}
}

- (void)setRate:(int)_rate
{
	rate = _rate;
	
	// apply pitch
	if ([parent class] == [AMSoundInstance class]) {
		FMOD::Channel *chan = *[(AMSoundInstance *)parent getFMODChannelPtr];
		
		// init base frequency if not already done
		if (baseFrequency == 0)
			chan->getFrequency(&baseFrequency);
		
		// set new rate
		chan->setFrequency((baseFrequency + (baseFrequency * (float)pitch / 12.0)) * (float)rate / 100000.0);
	}
}

- (void)applySettingsToInstance:(id)_instance
{
	if ([_instance isMemberOfClass:[AMSoundInstance class]]) {
		// get FMOD channel
		FMOD::Channel *chan = *[(AMSoundInstance *)_instance getFMODChannelPtr];
		
		// init base frequency if not already done
		if (baseFrequency == 0)
			chan->getFrequency(&baseFrequency);
		
		// set parameters
		chan->setFrequency((baseFrequency + (baseFrequency * (float)pitch / 12.0)) * (float)rate / 100000.0);
	}
}

- (void)setValue:(id)_value forAttribute:(int)_attribute {
	if (_attribute == AM_SND_PITCH) {
		self.pitch = [_value floatValue];
	} else if (_attribute == AM_SND_RATE) {
		self.rate = [_value floatValue];
	}
}

- (id)valueOfAttribute:(int)_attribute {
	if (_attribute == AM_SND_PITCH)
		return [NSNumber numberWithFloat:self.pitch];
	else if (_attribute == AM_SND_RATE)
		return [NSNumber numberWithFloat:self.rate];
	return nil;
}

- (int)attributeCode:(NSString *)_attribute {
	if ([AM_SOUND_CONTROL_ATTRIBUTE_PITCH isEqual:_attribute])
		return AM_SND_PITCH;
	else if ([AM_SOUND_CONTROL_ATTRIBUTE_RATE isEqual:_attribute])
		return AM_SND_RATE;
	return -1;
}

@end
