/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMTriggerControl.h
//  aria
//
//  Created by Yohan Lasorsa on 12/08/09.
//  Copyright 2009 INRIA, Team WAM. All rights reserved.
//

#import "AMControl.h"
#import "AMTime.h"
#import <Foundation/Foundation.h>


/* Constants of attributes names */
static const NSString *const AM_TRIGGER_CONTROL_ATTRIBUTE_CHANCE = @"chance";
static const NSString *const AM_TRIGGER_CONTROL_ATTRIBUTE_AUTOSTART = @"autoStart";

/** Attributes of the control. */
enum AMTriggerControlAttributes {
	AM_TRI_CHANCE = 0,
	AM_TRI_AUTOSTART
};


/**
 Allows to control the trigger frequency of the parent.
 */
@interface AMTriggerControl : AMControl {
	// Control attributes
	int chance;
	AMTime *autoStart;
}

/**
 The chance percentage that the parent will play each time a start action on it is triggered.
 Value must be between 0 and 100. Default value is 100.
 */
@property (nonatomic) int chance;

/**
 Defines an automatic trigger of a start action on the parent, with the given time interval.
 A nil value disables autoStart. Default value is nil.
 */
@property (nonatomic, copy) AMTime *autoStart;

@end
