/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMTriggerControl.mm
//  aria
//
//  Created by Yohan Lasorsa on 12/08/09.
//  Copyright 2009 INRIA, Team WAM. All rights reserved.
//

#import "AMTempoControl.h"
#import "AMControllableHierarchy.h"
#import "AMTriggerControl.h"


@implementation AMTriggerControl

@synthesize chance;
@synthesize autoStart;

- (id)init
{
	if (self = [super init]) {
		self.chance = 100;
		self.autoStart = nil;
	}
	return self;
}

- (void)setValue:(id)_value forAttribute:(int)_attribute {
	if (_attribute == AM_TRI_CHANCE)
		self.chance = [_value floatValue];
	else if (_attribute == AM_TRI_AUTOSTART)
		self.autoStart = [[[AMTime alloc] initWithTime:[[[AMValue alloc] initWithValue:[_value floatValue]] autorelease]] autorelease];
}

- (id)valueOfAttribute:(int)_attribute {
	if (_attribute == AM_TRI_CHANCE) {
		return [NSNumber numberWithFloat:self.chance];
	} else if (_attribute == AM_TRI_AUTOSTART) {
		int tempo = [(AMTempoControl *)[(id<AMControllableHierarchy>)parent getClosestControlWithName:AM_TEMPO_CONTROL] tempo];
		return (self.autoStart ? [NSNumber numberWithFloat:[self.autoStart calcTimeWithTempo:tempo]] : nil );
	}
	return nil;
}

- (int)attributeCode:(NSString *)_attribute {
	if ([AM_TRIGGER_CONTROL_ATTRIBUTE_CHANCE isEqual:_attribute])
		return AM_TRI_CHANCE;
	else if ([AM_TRIGGER_CONTROL_ATTRIBUTE_AUTOSTART isEqual:_attribute])
		return AM_TRI_AUTOSTART;
	return -1;
}

- (void)dealloc
{
	// release vars
	self.autoStart = nil;
	
	[super dealloc];
}

@end
