/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMAnimation.h
//  aria
//
//  Created by Yohan Lasorsa on 05/01/10.
//  Copyright 2010 INRIA, Team WAM. All rights reserved.
//

#import "AMPlayable.h"
#import "AMControllableHierarchy.h"
#import "AMIdentifiable.h"
#import "AMTime.h"
#import "AMValue.h"
#import "AMControl.h"
#import "AMAnimationInstance.h"
#import "a2ml.h"
#import <Foundation/Foundation.h>


/**
 An animation allows to create realtime variations of a specified control parameter.
 Animations are managed by the scheduler and can be synchronized on time and/or events.
 */
@interface AMAnimation : NSObject <AMPlayable, AMInstanciable, AMIdentifiable, AMControllableHierarchy> {
	// Animation attributes
	UID uid;
	AMTime *dur;
	AMTime *quantize;
	AMValue *from;
	AMValue *to;
	AMValue *by;
	AMFillType fill;
	AMAccumulationType accumulation;
	float repeatCount;
	AMTime *repeatDur;
	NSArray *values;
	NSString *eventValue;
	
	// Internal variables
	id<AMControllable> parent;		/* The parent element of the control to animate. */
	NSString *control;				/* The control to animate. */
	NSString *attribute;			/* The attribute to animate. */
	NSMutableArray *instances;		/* List of current instances of this cue. */
}

/**
 The duration of the animation.
 */
@property (nonatomic, retain) AMTime *dur;

/**
 The quantization value of the animation.
 */
@property (nonatomic, retain) AMTime *quantize;


/**
 The starting value of the parameter to animate.
 */
@property (nonatomic, retain) AMValue *from;

/**
 The ending value of the parameter to animate.
 */
@property (nonatomic, retain) AMValue *to;

/**
 The relative offset value of the parameter to animate.
 */
@property (nonatomic, retain) AMValue *by;

/**
 
 */
@property (nonatomic) AMFillType fill;

/**
 
 */
@property (nonatomic) AMAccumulationType accumulation;

/**
 
 */
@property (nonatomic) float repeatCount;

/**
 
 */
@property (nonatomic, retain) AMTime *repeatDur;

/**
 
 */
@property (nonatomic, retain) NSArray *values;

/**
 
 */
@property (nonatomic, retain) NSString *eventValue;


/**
 Creates a new animation for the specified control and parameter.
 @param parent The parent of the control to animate.
 @param control The control to animate.
 @param attribute The attribute of the control to animate.
 @return The new animation object.
 */
- (id)initWithParent:(id<AMControllable>)_parent control:(NSString *)_control attribute:(NSString *)_attribute;

/**
 Performs a static animation (set) with the given value.
 @param values The value to apply on the specified attribute of the animation.
 */
- (void)animateWithValues:(id)_value;


/**
 Returns the parent element of the control to animate.
 FOR INTERNAL USAGE ONLY.
 @return The parent element of the control to animate.
 */
- (id<AMControllable>)getControlParent;

/**
 Returns the name of the control to animate.
 FOR INTERNAL USAGE ONLY.
 @return The name of the control to animate.
 */
- (NSString *)getControlName;

/**
 Returns the attribute of the control to animate.
 FOR INTERNAL USAGE ONLY.
 @return The attribute of the control to animate.
 */
- (NSString *)getAttributeName;

@end
