/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMSection.h
//  aria
//
//  Created by Yohan Lasorsa on 01/07/09.
//  Copyright 2009 INRIA, Team WAM. All rights reserved.
//

#import "AMIdentifiable.h"
#import "AMControllable.h"
#import "AMCue.h"
#import "a2ml.h"
#import "fmod.hpp"
#import "fmod_errors.h"
#import <Foundation/Foundation.h>


/**
 A section allow to define separate audio groups for mixing.
 Additional DSP effects can be added, and the mix of a section can be done either in 2D or 3D.
 */
@interface AMSection : NSObject <AMControllable, AMIdentifiable, AMControllableHierarchy>  {
	// Section attributes
	UID uid;
	NSMutableDictionary *controls;
	
	// Internal variables
	FMOD::ChannelGroup *group;			/* The FMOD channel group of this section. */	
}

/**
 Creates a new section object.
 @return The created section object.
 */
- (id)init;

/**
 Adds a cue to the section.
 @param cue The AMChue object to add.
 */
- (void)addCue:(AMCue *)_cue;

/**
 Removes the cue from the section.
 @param idx The index of the chunk object to remove.
 */
- (void)removeCue:(AMCue *)_cue;


/**
 Returns the internal FMOD channel group.
 FOR INTERNAL USAGE ONLY.
 @return The internal FMOD channel group.
 */
- (FMOD::ChannelGroup *)getFMODChannelGroup;

@end
