/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMSection.mm
//  aria
//
//  Created by Yohan Lasorsa on 01/07/09.
//  Copyright 2009 INRIA, Team WAM. All rights reserved.
//

#import "AMSection.h"


@implementation AMSection

- (id)init
{
	if (self = [super init]) {
		uid = GenerateUniqueID();
		controls = nil;
		
		[AMSoundManager getFMODSystem]->createChannelGroup(NULL, &group);
	}
	return self;
}

- (UID)getUID
{
	return uid;
}

- (void)addCue:(AMCue *)_cue
{
	if (_cue != nil) {
		NSMutableArray **cgroups = [_cue getFMODChannelGroups];
		
		if (*cgroups == nil)
			*cgroups = [[NSMutableArray alloc] initWithCapacity:1];		// TODO: optimize this?

		[*cgroups addObject:[NSValue valueWithPointer:group]];
	}
}

- (void)removeCue:(AMCue *)_cue
{
	if (_cue != nil) {
		NSMutableArray **cgroups = [_cue getFMODChannelGroups];
		
		if (*cgroups != nil) {
			[*cgroups removeObject:[NSValue valueWithPointer:group]];
			
			if ([*cgroups count] == 0) 
				*cgroups = nil;
		}
	}
}

- (NSSet *)getControls
{
	return [NSSet setWithObjects:AM_MIX_2D_CONTROL, AM_MIX_3D_CONTROL, nil];
}

- (AMControl *)getControlWithName:(NSString *)_name
{
	return [controls objectForKey:_name];
}

- (AMControl *)createControlWithName:(NSString *)_name
{
	AMControl *control = [self getControlWithName:_name];
	
	// if the control has not been instanciated
	if (control == nil) {
		control = [AMControl controlWithName:_name controllable:self];
		
		// if the control is valid
		if (control != nil) {
			if (controls == nil)
				controls = [[NSMutableDictionary alloc] initWithCapacity:AM_CONTROLS_INITIAL_CAPACITY];						
			[controls setObject:control forKey:_name];
		}
	}
	
	return control;
}

- (AMControl *)getClosestControlWithName:(NSString *)_name
{
	AMControl *control = [controls objectForKey:_name];
	
	if (control == nil) {
		control = [AMSoundManager createControlWithName:_name];
	} else {
		// keeps the object valid if the section gets killed.
		[[control retain] autorelease];
	}
	
	return control;
}

- (FMOD::ChannelGroup *)getFMODChannelGroup
{
	return group;
}

-(void)dealloc 
{
	// release vars
	if (controls != nil)
		[controls release];
	group->release();
	
	AM_LOG_CORE(@"Released.");	
	
	[super dealloc];
}

@end
