/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMChunkInstance.h
//  aria
//
//  Created by Yohan Lasorsa on 03/07/09.
//  Copyright 2009 INRIA, Team WAM. All rights reserved.
//

#import "AMInstance.h"
#import "AMChunk.h"
#import "AMPlayable.h"
#import "AMSoundListener.h"
#import "AMSoundInstance.h"
#import "AMListenable.h"
#import "AMChunkListener.h"
#import <Foundation/Foundation.h>


// to avoid import loop errors
@class AMSoundInstance;

/**
 A chunk element instance.
 Used internally by the AMChunk class.
 */
@interface AMChunkInstance : AMInstance <AMSoundListener, AMPlayable> {
	// Internal variables
	int activeSound;				/* Index of the currently active sound. */
	int nextSound;					/* Index of the next sound to be played. */	
	AMSoundInstance *sndInstance;	/* The instance of the sound currently playing, if applicable. */
}

/**
 Updates the next sound to be played.
 @return The index of the next sound to be played.
 */
- (int)updateNextSound;


/**
 Returns the internal sound instance of this chunk.
 FOR INTERNAL USE ONLY.
 @return The internal sound instance of this chunk.
 */
- (AMSoundInstance *)getSoundInstance;

@end
