/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMChunkInstance.mm
//  aria
//
//  Created by Yohan Lasorsa on 03/07/09.
//  Copyright 2009 INRIA, Team WAM. All rights reserved.
//

#import "AMChunkInstance.h"


@implementation AMChunkInstance

/*
- (id)initWithParent:(id)_parent
{
	return [self initWithParent:_parent id:GenerateUniqueID()];
}*/

- (id)initWithParent:(id)_parent parentUid:(UID)_parentUid
{
	if (self = [super initWithParent:_parent parentUid:_parentUid]) {		
		if (parent != nil) {
			int next = [parent nextSound];
			if (next != -1 && [(AMChunk *)parent pick] != AM_PT_FIXED)
				activeSound = next;
			else
				activeSound = [parent activeSound];
			[self updateNextSound];
			sndInstance = nil;
		}
	}
	return self;
}

/*
 Internal function.
 @param excludeCurrent Set YES to exclude the current active sound from the random selection, NO to include it.
 @return A randomly chosen sound.
 */
- (int)getRandomSound:(BOOL)_excludeCurrent
{
	NSArray *sounds = [(AMChunk *)parent getSounds];
	int count = [sounds count];
	int idx = 0;	// kepp index 0 if there is 1 or no sound
	
	if (count == 2)	{
		if (_excludeCurrent) {
			if (activeSound == 0)
				idx = 1;
			else
				idx = 0;
		} else {
			int probaThreshold = [[sounds objectAtIndex:0] pickPriority];
			int probaSum = probaThreshold + [[sounds lastObject] pickPriority];
			
			// generate random number and pick corresponding sound
			if (RAND_INT(probaSum) < probaThreshold)
				idx = 0;
			else
				idx = 1;
		}
	} else if (count > 2) {
		int i = 0, probaSum = 0, rnd;
		int probaTable[count];
		
		// generate the probability table
		for (i = 0; i < count; i++) {
			if (!_excludeCurrent || i != activeSound) {
				probaSum += [[sounds objectAtIndex:i] pickPriority];
			}
			probaTable[i] = probaSum;
		}
		
		// generate random number and pick corresponding sound
		rnd = RAND_INT(probaSum);
		i = 0;
		while (rnd >= probaTable[i]) {
			i++;
		}
		
		idx = i;
	}
	
	return idx;
}

- (int)updateNextSound
{
	NSArray *sounds = [(AMChunk *)parent getSounds];
	int count;
	
	switch ([(AMChunk *)parent pick]) {
		case AM_PT_RANDOM:
			nextSound = [self getRandomSound:NO];
			break;
		case AM_PT_OMIT_MOST_RECENT:
			nextSound = [self getRandomSound:YES];
			break;
		case AM_PT_REVERSED:
			count = [sounds count];
			if (count > 0) {
				nextSound = activeSound - 1;
				if (nextSound < 0)
					nextSound = count - 1;
			} else {
				nextSound = 0;
			}
			break;
		case AM_PT_FIXED:
			nextSound = activeSound;
			break;
		default:	// AM_PT_ORDERED
			count = [sounds count];
			if (count > 0) {
				nextSound = activeSound + 1;
				if (nextSound >= count)
					nextSound = 0;
			} else {
				nextSound = 0;
			}
			break;
	}
	
	// also update parent chunk next sound
	((AMChunk *)parent).nextSound = nextSound;
	
	return nextSound;
}

/*
 Internal function.
 Updates the active sound duration by specifying a stopping time with the set duration of the chunk.
 */ /*
- (void)updateDur
{
	
}*/

- (id)start
{
	// only start if it's not already playing
	if (sndInstance == nil && activeSound < [[parent getSounds] count]) {
		AMSound *sound = [[parent getSounds] objectAtIndex:activeSound];
		sndInstance = [sound startFromParentUid:parentUid listener:self];
	}
	
	return self;
}

- (id)stop
{
	if (sndInstance != nil) {
		[sndInstance stop];
		sndInstance == nil;
	}
	
	return self;
}

- (BOOL)isPlaying
{
	if (sndInstance != nil)
		return [sndInstance isPlaying];
	return NO;	
}

- (double)getDuration
{
	if (sndInstance != nil)
		return [sndInstance getDuration];
	
	return [[[parent getSounds] objectAtIndex:activeSound] getDuration];
}

- (void)soundUpdate:(AMSound *)_sound eventType:(AMSoundEventType)_type eventData:(void *)_data
{
	switch (_type) {
		case AM_SND_STARTED:
		{
			[(id <AMChunkListener>)listener chunkUpdate:parent eventType:AM_CHK_STARTED eventData:NULL];
			
			AM_LOG_CORE(@"[ID:%lld, PID:%lld] Sound %d was picked.", uid, parentUid, activeSound+1);

			// send notification
			NSString *event = [AMSoundManager getIDFromKeyUID:[parent getUID]];			
			if (event) {
				event = [event stringByAppendingString:AM_PLAYABLE_BEGIN_SUFFIX];
				
				[[AMSoundManager getNotificationCenter] postNotificationName:event object:[[self retain] autorelease]];
				
				// TODO:send also instance event
			}
			
			break;
		}
		case AM_SND_STOPPED:
		{
			sndInstance = nil;
			[(id <AMChunkListener>)listener chunkUpdate:parent eventType:AM_CHK_STOPPED eventData:NULL];
			
			// send notification
			NSString *event = [AMSoundManager getIDFromKeyUID:[parent getUID]];			
			if (event) {
				event = [event stringByAppendingString:AM_PLAYABLE_END_SUFFIX];
				
				[[AMSoundManager getNotificationCenter] postNotificationName:event object:[[self retain] autorelease]];
				
				// TODO:send also instance event
			}
			
			break;
		}
		case AM_SND_ENDED:
		{
			sndInstance = nil;

			// update current sound
			activeSound = nextSound;
			[self updateNextSound];
			
			[(id <AMChunkListener>)listener chunkUpdate:parent eventType:AM_CHK_ENDED eventData:NULL];
			
			// send notification
			NSString *event = [AMSoundManager getIDFromKeyUID:[parent getUID]];			
			if (event) {
				event = [event stringByAppendingString:AM_PLAYABLE_END_SUFFIX];
				
				[[AMSoundManager getNotificationCenter] postNotificationName:event object:[[self retain] autorelease]];
				
				// TODO:send also instance event
			}
			
			break;
		}
		default:
			break;
	}
}

- (AMSoundInstance *)getSoundInstance
{
	return sndInstance;
}

-(void)dealloc 
{	
	AM_LOG_CORE(@"[ID:%lld, PID:%lld] Released.", uid, parentUid);
	
	[super dealloc];
}

@end
