/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMCueInstance.h
//  aria
//
//  Created by Yohan Lasorsa on 03/07/09.
//  Copyright 2009 INRIA, Team WAM. All rights reserved.
//

#import "AMInstance.h"
#import "AMChunkListener.h"
#import "AMChunkInstance.h"
#import "AMListenable.h"
#import "AMCue.h"
#import "AMCueListener.h"
#import "AMControl.h"
#import "AMControllable.h"
#import "a2ml.h"
#import <Foundation/Foundation.h>


// to avoid import loop errors
@class AMChunkInstance;

/**
 A cue element instance.
 Used internally by the AMCue class.
 */
@interface AMCueInstance : AMInstance <AMControllable, AMChunkListener, AMPlayable> {
	// Internal variables
	int currentChunk;				/* Index of the currently active chunk. */
	int nextChunk;					/* Index of the next chunk to be played. */
	int remainingLoops;				/* Remaining number of time to play. */	
	int chunksPlaying;				/* Number of chunks currently playing. */
	char releaseTimeout;			/* Timeout counter for releasing this instance. */
	AMChunkInstance *chkInstance1;	/* The instance of the currently playing chunk, if applicable. */
	AMChunkInstance *chkInstance2;	/* The instance of the fading in chunk, if applicable. */
	NSMutableDictionary *controls;	/* The instanciable controls. */
}

/**
 Timeout counter for releasing this instance.
 INTERNAL USAGE ONLY.
 */
@property (nonatomic) char releaseTimeout;

/**
 Updates the next chunk to be played.
 @return The index of the next chunk to be played.
 */
- (int)updateNextChunk;


/**
 Returns the 1st internal chunk instance of this cue.
 FOR INTERNAL USE ONLY.
 @return The 1st internal chunk instance of this cue.
 */
- (AMChunkInstance *)getChunkInstance1;

/**
 Returns the 2nd internal chunk instance of this cue.
 FOR INTERNAL USE ONLY.
 @return The 2nd internal chunk instance of this cue.
 */
- (AMChunkInstance *)getChunkInstance2;

@end
