/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMInstance.h
//  aria
//
//  Created by Yohan Lasorsa on 11/08/09.
//  Copyright 2009 INRIA, Team WAM. All rights reserved.
//

#import "a2ml.h"
#import "AMInstanciable.h"
#import "AMListenable.h"
#import "AMControl.h"
#import "AMControllableHierarchy.h"
#import "AMIdentifiable.h"
#import "AMParentIdentifiable.h"
#import <Foundation/Foundation.h>


/**
 A generic object instance.
 This class is not destined to be used directly, but to be derived.
 */
@interface AMInstance : NSObject <AMListenable, AMIdentifiable, AMParentIdentifiable, AMControllableHierarchy> {
	// Instance attributes
	UID uid;
	UID parentUid;
	id parent;
	id listener;
}

/**
 The parent object of the instance.
 */
@property (nonatomic, readonly) id parent;

/**
 The optional listener for the instance's events.
 */
@property (nonatomic, assign) id listener;


/**
 Creates a new instance object.
 @param parent The parent object of the instance. The parent must implement the AMInstanciable protocol.
 @param parentUid The unique ID of the object holding this instance (usually the upper instance in the hierarchy).
 @return A new instance object.
 */
- (id)initWithParent:(id)_parent parentUid:(UID)_parentUid;

/**
 Kills the instance.
 Needs to be overriden in derived classes!
 The message destroyInstance is sent to the parent with this instance as the target.
 */
- (void)kill;

@end
