/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMSoundInstance.h
//  aria
//
//  Created by Yohan Lasorsa on 03/07/09.
//  Copyright 2009 INRIA, Team WAM. All rights reserved.
//

#import "a2ml.h"
#import "AMPlayable.h"
#import "AMInstance.h"
#import "AMControl.h"
#import "AMSound.h"
#import "AMSoundListener.h"
#import "AMSoundManager.h"
#import "fmod.hpp"
#import "fmod_errors.h"
#import <Foundation/Foundation.h>


/**
 A sound element instance.
 Used internally by the AMSound class.
 */
@interface AMSoundInstance : AMInstance <AMControllable, AMPlayable> {
	// Internal variables
	int remainingLoops;				/* Number of remaining loops to play. */
	BOOL stopped;					/* YES is the sound instance was stopped. */
	FMOD::Channel *channel;			/* The FMOD channel used to play the sound. */
	NSMutableDictionary *controls;	/* The instanciable controls. */
}


/**
 The internal number of remaining loops to play.
 FOR INTERNAL USAGE ONLY.
 */
@property (nonatomic) int remainingLoops;

/**
 Set to YES if the sound instance was stopped.
 FOR INTERNAL USAGE ONLY.
 */
@property (nonatomic) BOOL stopped;

/**
 Returns a pointer to the internal FMOD channel object.
 FOR INTERNAL USAGE ONLY.
 @return The internal FMOD channel object pointer.
 */
- (FMOD::Channel **)getFMODChannelPtr;

@end
