/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMSoundListener.h
//  aria
//
//  Created by Yohan Lasorsa on 18/08/09.
//  Copyright 2009 INRIA, Team WAM. All rights reserved.
//

#import "AMSound.h"
#import <Foundation/Foundation.h>


// to avoid import loop errors
@class AMSound;

/**
 The sound events that may be triggered.
 */
enum AMSoundEventType {
	AM_SND_STARTED = 0,				// The sound started
	AM_SND_STOPPED,					// The sound was stopped
	AM_SND_ENDED,					// The sound ended
	AM_SND_SETACTIVE_REQUEST		// A request was to set this sound as the active one in its parent chunk
};

/**
 Provides a protocol for receiving events from sound objects.
 */
@protocol AMSoundListener

/**
 Receives a event from a sound objet.
 @param sound The sound that triggered the event.
 @param type The type of the event that was triggered.
 @param data Optional data that may be transmitted with the event.
 */
- (void)soundUpdate:(AMSound *)_sound eventType:(AMSoundEventType)_type eventData:(void *)_data;

@end
