/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMParser.h
//  aria
//
//  Created by Yohan Lasorsa on 01/07/09.
//  Copyright 2009 INRIA, Team WAM. All rights reserved.
//

#import "AMSection.h"
#import "AMCue.h"
#import "AMChunk.h"
#import "AMSound.h"
#import "AMTempoControl.h"
#import "AMTriggerControl.h"
#import "AMSpawnControl.h"
#import "AMMix2DControl.h"
#import "AMMix3DControl.h"
#import "AMSoundControl.h"
#import "a2ml.h"
#import <Foundation/Foundation.h>


/**
 Loads an A2ML file and initialize objects and structures from it.
 */
@interface AMParser : NSObject <NSXMLParserDelegate> {
	// Internal variables
	NSString *basePath;
	id<AMControllable> lastControllable;
	id<AMPlayable> lastPlayable;
	NSString *lastControl;
	AMCue *lastCue;
	AMChunk *lastChunk;
	AMSound *lastSound;
	BOOL inMasterSection;
}

/**
 Parses an A2ML document and initialize the audio structures and scheduler interactions from it.
 @param URL The URL of the A2ML document to parse.
 @param error If the parsing failed at some point, gets the NSError describing the problem.
 */
- (void)parseA2MLFileAtURL:(NSString *)_URL parseError:(NSError **)_error;

@end
