/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMControllable.h
//  aria
//
//  Created by Yohan Lasorsa on 01/07/09.
//  Copyright 2009 INRIA, Team WAM. All rights reserved.
//

#import "AMControl.h"
#import <Foundation/Foundation.h>


// to avoid import loop
@class AMControl;

/**
 Provides a protocol for retrieving controls from controllable objects.
 */
@protocol AMControllable

/**
 Gets the name of the controls supported.
 @return An NSSet object containing the names of the controls as NSString objects.
 */
- (NSSet *)getControls;

/**
 Gets the control with the specified name.
 @param name The name of the control to get. Should be one of the constant control names defined in the AMControl class.
 @return The desired control, or nil if it does not exist.
 */
- (AMControl *)getControlWithName:(NSString *)_name;

/**
 Gets the control with the specified name or creates it if it doesn't exist.
 @param name The name of the control to get or create. Should be one of the constant control names defined in the AMControl class.
 @return The desired control, or nil if it is not supported by the audio object.
 */
- (AMControl *)createControlWithName:(NSString *)_name;

@end
