/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMInstanciable.h
//  aria
//
//  Created by Yohan Lasorsa on 11/08/09.
//  Copyright 2009 INRIA, Team WAM. All rights reserved.
//

#import "a2ml.h"
#import <Foundation/Foundation.h>


/**
 Provides a protocol for instanciable audio objects.
 */
@protocol AMInstanciable

/** 
 Creates a new instance of the object if possible and return it.
 If creating a new instance is not possible, nil is returned.
 @return The new instance object or nil.
 */
- (id)createInstance;

/** 
 Creates a new instance of the object with the given parent UID if possible and return it.
 If creating a new instance is not possible, nil is returned.
 @param uid The UID of the object holding the instance new instance.
 @return The new instance object or nil.
 */
- (id)createInstanceWithParentUid:(UID)_uid;

/**
 Destroy the given instance of the object.
 @param instance The instance of the object to destroy.
 */
- (void)destroyInstance:(id)_instance;

/**
 Destroy the instance of the playable source with the given parent UID.
 @param uid The parent UID you want to destroy the corresponding instance.
 */
- (void)destroyInstanceFromParentUid:(UID)_uid;

/**
 Starts the playable source with the given parent UID.
 @param uid The UID of the object holding the instance of the playable object.
 @param listener The optional listener of the created instance events.
 @return The playing instance of the source.
 */
- (id)startFromParentUid:(UID)_uid listener:(id)_listener;

/**
 Stops the instance of the playable source with the given parent UID.
 If no instance is found with this ID suffix, it will be ignored.
 @param uid The parent UID of the instance to stop.
 @return The instance of the source that was stopped.
 */
- (id)stopFromParentUid:(UID)_uid;

/**
 Gets the current playing status of the instance of the source with the given parent UID.
 @param uid The parent UID of the instance.
 @return YES if the source is currently playing, NO otherwise.
 */
- (BOOL)isPlayingFromParentUid:(UID)_uid;

/**
 Gets the duration of the instance of the playable source with the given parent UID.
 @param uid The parent UID of the instance.
 @return The duration of the sources in seconds, or -1 if the duration is unknown.
 */
- (double)getDurationFromParentUid:(UID)_uid;

@end
