/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMRandomValue.h
//  aria
//
//  Created by Yohan Lasorsa on 06/08/09.
//  Copyright 2009 INRIA, Team WAM. All rights reserved.
//

#import "a2ml.h"
#import "AMValue.h"
#import <Foundation/Foundation.h>


/**
 A generic randomized value.
 */
@interface AMRandomValue : AMValue <NSCopying> {
	// Internal variables
	double minValue;
	double maxValue;
	double interval;
}

@property (nonatomic) double minValue;				/**< The minimum value. */
@property (nonatomic) double maxValue;				/**< The maximum value. */
@property (nonatomic) double interval;				/**< The interval between random generated values. */

/** 
 Creates a new random value object.
 @param minValue The minimum generated value.
 @param maxValue The upper limit for the generated value.
 @param interval The interval between generated values.
 @return The created random value object. 
 */
- (id)initWithMinValue:(double)_minValue maxValue:(double)_maxValue interval:(double)_interval;

/** 
 Creates a new random value object.
 @param minValue The minimum generated value.
 @param maxValue The upper limit for the generated value.
 @return The created random value object. 
 */
- (id)initWithMinValue:(double)_minValue maxValue:(double)_maxValue;

/**
 Generates a new value.
 The interval value returned by the getValue() function is also updated.
 @return The generated value. 
 */
- (double)calcValue;

@end
