/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMRandomValue.mm
//  aria
//
//  Created by Yohan Lasorsa on 06/08/09.
//  Copyright 2009 INRIA, Team WAM. All rights reserved.
//

#import "AMRandomValue.h"


@implementation AMRandomValue

@synthesize minValue;
@synthesize maxValue;
@synthesize interval;

- (id)initWithMinValue:(double)_minValue maxValue:(double)_maxValue
{
	return [self initWithMinValue:_minValue maxValue:_maxValue interval:0];
}

- (id)initWithMinValue:(double)_minValue maxValue:(double)_maxValue interval:(double)_interval
{
	if (self = [super init]) {
		minValue = _minValue;
		maxValue = _maxValue;
		interval = _interval;
	}
	return self;
}

- (double)calcValue 
{	
	value = minValue + RAND(maxValue - minValue);

	if (interval != 0) {
		double rest = fmod(value, interval);
		value -= rest;
		if (value < minValue)
			value += interval;
	}
	
	return value;
}

- (id)copyWithZone:(NSZone *)zone
{
    AMRandomValue *copy = [[[self class] allocWithZone: zone] initWithMinValue:minValue maxValue:maxValue interval:interval];
    return copy;
}

@end
