/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMTime.h
//  aria
//
//  Created by Yohan Lasorsa on 29/06/09.
//  Copyright 2009 INRIA, Team WAM. All rights reserved.
//

#import "a2ml.h"
#import "AMValue.h"
#import "AMRandomValue.h"
#import <Foundation/Foundation.h>


/**
 A time representation, based on standard time or tempo clock source.
 */
@interface AMTime : NSObject <NSCopying> {
	// Internal variables
	AMValue *time;
	BOOL isTimeTempoBased;
}

@property (nonatomic, copy) AMValue *time;		/**< The base time value, in seconds or ticks. */
@property (nonatomic) BOOL isTimeTempoBased;	/**< Indicate whether the time value is standard (seconds) or tempo-based (ticks). */

/** 
 Creates a new time object.
 @param time The time value in seconds.
 @return The created time object. 
 */
- (id)initWithTime:(AMValue *)_time;

/** 
 Creates a new time object.
 @param time The time value in seconds or ticks.
 @param isTimeTempoBased Specifies if the time value is tempo-based or not.
 @return The created time object. 
 */
- (id)initWithTime:(AMValue *)_time isTimeTempoBased:(BOOL)_isTimeTempoBased;

/**
 Calculates the resulting time value, taking into account the random offsets that may apply.
 If tempo-based values are used, the global tempo clock of the sound manager will be used.
 @return The time value in seconds. 
 */
- (double)calcTime;

/**
 Calculates the resulting time value, taking into account the random offsets that may apply.
 @param tempo The tempo to use to calculate tempo-based values.
 @return The time value in seconds. 
 */
- (double)calcTimeWithTempo:(double)_tempo;

/**
 Calculates the resulting time value, taking into account the random offsets that may apply.
 @param tempo The tempo to use to calculate tempo-based values.
 @param quantizer The quantization value that will be used.
 @param isQuantizerTempoBased Specifies if the quantization value is tempo-based or not.
 @return The time value in seconds. 
 */
- (double)calcTimeWithTempo:(double)_tempo quantizer:(double)_quantizer isQuantizerTempoBased:(BOOL)_isQuantizerTempoBased;

@end
