/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMTime.mm
//  aria
//
//  Created by Yohan Lasorsa on 29/06/09.
//  Copyright 2009 INRIA, Team WAM. All rights reserved.
//

#import "AMTime.h"


@implementation AMTime

@synthesize time;
@synthesize isTimeTempoBased;

- (id)initWithTime:(AMValue *)_time 
{
	return [self initWithTime:_time isTimeTempoBased:NO];
}

- (id)initWithTime:(AMValue *)_time isTimeTempoBased:(BOOL)_isTimeTempoBased 
{
	if (self = [super init]) {
		time = [_time copy];
		isTimeTempoBased = _isTimeTempoBased;
	}
	return self;
}

- (double)calcTime 
{
	//TODO: get tempo from sound manager
	// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	double bpm = AM_DEFAULT_TEMPO;
	
	return [self calcTimeWithTempo:bpm];
}

- (double)calcTimeWithTempo:(double)_tempo
{
	return [self calcTimeWithTempo:_tempo quantizer:0 isQuantizerTempoBased:NO];
}

- (double)calcTimeWithTempo:(double)_tempo quantizer:(double)_quantizer isQuantizerTempoBased:(BOOL)_isQuantizerTempoBased;
{
	double result = 0;
	double align = _quantizer;
	double bpm = _tempo;

	// calculate the time value if randomization is used
	result = [time calcValue];

	// check if the tempo is valid
	if (bpm <= 0)
		bpm = AM_DEFAULT_TEMPO;
		
	// do the tempo=>time conversions if needed
	if (isTimeTempoBased)
		result /= bpm * AM_TEMPO_RESOLUTION / 60;
	
	// apply quantization if needed
	if (_quantizer != 0) {
		if (_isQuantizerTempoBased) {
			align /= bpm * AM_TEMPO_RESOLUTION / 60;
		}
		
		double rest = fmod(result, align);
		if (rest > 0) {
			result += align - rest;
		}
	}
	
	return result;
}

- (id)copyWithZone:(NSZone *)zone
{
    AMTime *copy = [[[self class] allocWithZone: zone] initWithTime:time isTimeTempoBased:isTimeTempoBased];
    return copy;
}

-(void)dealloc
{
	// release vars
	[time release];
	
	[super dealloc];
}

@end
