/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMEventAction.h
//  aria
//
//  Created by Yohan Lasorsa on 05/01/10.
//  Copyright 2010 INRIA, Team WAM. All rights reserved.
//

#import "AMPlayable.h"
#import "AMTime.h"
#import "a2ml.h"
#import <Foundation/Foundation.h>


/**
 This class allows to perform an action on a playable object, based on an event trigger.
 @see AMScheduler
 @see AMPlayable
 */
@interface AMEventAction : NSObject {
	// Attributes
	NSString *event;
	id playable;
	AMActionType action;
	AMTime *delay;
	BOOL permanent;
}

/**
 Creates a new permanent or single-use AMEventAction object.
 @param playable The playable object on which the action will occur.
 @param event The event that will trigger the action.
 @param action The action to perform, as defined by the AMActionType enumeration.
 @param delay The delay before executing the action after the specified event has been received.
 @param permanent If set to YES, this AMTimeAction object will be dismissed by the scheduler after having performed the action.
 @return The created AMTimeAction object.
 */
- (id)initWithPlayable:(id)_playable event:(NSString *)_event action:(AMActionType)_action delay:(AMTime *)_delay permanent:(BOOL)_permanent;

/**
 Gets the event trigger of the action.
 @return The event that will trigger the action, or nil if it was a single-use action that has already been performed.
 */
- (NSString *)getEvent;

/**
 Performs the registered action on the linked playable object.
 Automatically registered as an observer of the target event when the AMEventAction object is created.
 @param notification The event notification containing the event name and the optional data associated with the event, that may be used for the action.
 */
- (void)doAction:(NSNotification *)_notification;

@end
