/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMEventAction.mm
//  aria
//
//  Created by Yohan Lasorsa on 05/01/10.
//  Copyright 2010 INRIA, Team WAM. All rights reserved.
//

#import "AMTempoControl.h"
#import "AMControllableHierarchy.h"
#import "AMEventAction.h"
#import "AMScheduler.h"
#import "AMSoundManager.h"


@implementation AMEventAction

- (id)initWithPlayable:(id)_playable event:(NSString *)_event action:(AMActionType)_action delay:(AMTime *)_delay permanent:(BOOL)_permanent
{
	if (self = [super init]) {
		playable = [_playable retain];
		event = [_event retain];
		action = _action;
		delay = [_delay retain];
		permanent = _permanent;
	}

	[[AMSoundManager getNotificationCenter] addObserver:self selector:@selector(doAction:) name:event object:nil];

	return self;
}

- (NSString *)getEvent
{
	return event;
}

- (void)doAction:(NSNotification *)_notification
{
	if (delay > 0 && action != AM_ACT_SET) {
		// delay the action
		AMScheduler *scheduler = [AMSoundManager getScheduler];
		int tempo = [(AMTempoControl *)[(id<AMControllableHierarchy>)playable getClosestControlWithName:AM_TEMPO_CONTROL] tempo];
		AMValue *val;
		
		if ((action == AM_ACT_START || action == AM_ACT_STOP) && [playable respondsToSelector:@selector(quantize)]) {
			AMTime *quantizer = [playable quantize];
			val = [[AMValue alloc] initWithValue:([scheduler currentTime] + [delay calcTimeWithTempo:tempo quantizer:[quantizer calcTimeWithTempo:tempo] isQuantizerTempoBased:[quantizer isTimeTempoBased]])];
		} else {
			val = [[AMValue alloc] initWithValue:([scheduler currentTime] + [delay calcTimeWithTempo:tempo])];
		}
		
		[val autorelease];
		AMTime *time = [[AMTime alloc] initWithTime:val];
		[time autorelease];
		
		[scheduler registerTimeAction:time playable:playable action:action permanent:NO];
		
	} else {
		// do the action
		switch (action) {
			case AM_ACT_START:
			{
				[(id <AMPlayable>)playable start];

				//				long dur = parent.getDuration();
				//				parent.start();
				//				if (dur >= 0 && (parent instanceof Cue)) {  // only needed for cues, animations stop themselves
				//					((Cue)parent).addCueListener(SoundManager.getScheduler().registerTimeAction(currentTime + dur, parent, Action.STOP, false));
				//				}
				break;
			}
				
			case AM_ACT_STOP:
				[(id <AMPlayable>)playable stop];
				break;
				
			case AM_ACT_SET:
				
				AM_LOG(@"Animation set received, with value: %@", [_notification object]);
				
				if ([playable isMemberOfClass:[AMAnimation class]]) {
					AMAnimation *anim = (AMAnimation *)playable;
					[anim animateWithValues:[_notification object]];
				}
				break;
				
			case AM_ACT_ENABLE:
				// TODO
				//				((Chunk)parent).setEnabled(true);
				break;
				
			case AM_ACT_DISABLE:
				// TODO
				//				((Chunk)parent).setEnabled(false);
				break;
				
			case AM_ACT_SET_ACTIVE:
				[(AMSound *)playable setActive];
				break;
				
			default:
				break;
		}
	}
		
	// If the action was intented to be single-use, dismiss it.
	if (!permanent) {
		[[AMSoundManager getNotificationCenter] removeObserver:self];

		[event release];
		event = nil;

		[[AMSoundManager getScheduler] removeEventAction:self];
	}
}


 -(void)dealloc 
{
	AM_LOG_SCHEDULER(@"dealloc with playable [ID:%lld]", [playable getUID]);

	[[AMSoundManager getNotificationCenter] removeObserver:self];
	
	// release vars
	[playable release];
	[event release];
	[delay release];
	
	[super dealloc];
}


@end
