/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMTimeAction.h
//  aria
//
//  Created by Yohan Lasorsa on 05/01/10.
//  Copyright 2010 INRIA, Team WAM. All rights reserved.
//

#import "AMTime.h"
#import "AMPlayable.h"
#import "a2ml.h"
#import <Foundation/Foundation.h>


/**
 * This class allows to perform an action on a playable object, and is linked to a time
 * interval relative to the scheduler initial time.
 * @see AMScheduler
 * @see AMPlayable
 */
@interface AMTimeAction : NSObject {
	// Attributes
	id playable;
	AMActionType action;
	double begin;
	BOOL permanent;
}

/**
 Creates a new permanent or single-use AMTimeAction object.
 @param playable The playable object on which the action will occur.
 @param begin The scheduled begin time for the action to perform, in seconds.
 @param action The action to perform, as defined by the AMActionType enumeration.
 @param permanent If set to YES, this AMTimeAction object will be dismissed by the scheduler after having performed the action.
 @return The created AMTimeAction object.
 */
- (id)initWithPlayable:(id)_playable begin:(double)_begin action:(AMActionType)_action permanent:(BOOL)_permanent;

/**
 Gets the scheduled begin time of the action.
 @return The begin time of the action in seconds, or -1 if it was a single-use action that has already been performed.
 */
- (double)getBeginTime;

/**
 Performs the registered action on the linked playable object.
 @param currentTime The current time given of the scheduler time base, in seconds.
 @param timeInterval The time interval that has to be considerated as current, negatively relative to the currentTime parameter, in seconds.
 @return YES if the action was performed and the AMTimeAction was set to be single-use, NO otherwise. 
 */
- (BOOL)doAction:(double)_currentTime timeInterval:(double)_timeInterval;

@end
