/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  AMTimeAction.mm
//  aria
//
//  Created by Yohan Lasorsa on 05/01/10.
//  Copyright 2010 INRIA, Team WAM. All rights reserved.
//

#import "AMTempoControl.h"
#import "AMControllableHierarchy.h"
#import "AMTimeAction.h"
#import "AMCueInstance.h"
#import "AMSoundManager.h"


@implementation AMTimeAction

- (id)initWithPlayable:(id)_playable begin:(double)_begin action:(AMActionType)_action permanent:(BOOL)_permanent
{
	if (self = [super init]) {
		playable = [_playable retain];
		action = _action;
		begin = _begin;
		permanent = _permanent;
	}
	return self;
}

- (double)getBeginTime
{
	return begin;
}

- (BOOL)doAction:(double)_currentTime timeInterval:(double)_timeInterval
{
	BOOL dismiss = false;
	
	if (_currentTime - _timeInterval < begin && begin <= _currentTime) {
		// Do action if we're in the right time interval.
		switch (action) {
			case AM_ACT_START:
			{
				AM_LOG(@"Time action started.");
				
				id instance = [(id<AMPlayable>)playable start];
				
				if ([instance class] == [AMCueInstance class]) {
					AMTime *durTime = ((AMCue *)playable).dur;

					if (durTime != nil) {
						int tempo = [(AMTempoControl *)[(id<AMControllableHierarchy>)playable getClosestControlWithName:AM_TEMPO_CONTROL] tempo];
						double quantizer = [((AMCue *)playable).quantize calcTimeWithTempo:tempo];
						BOOL isQTempoBased = [((AMCue *)playable).quantize isTimeTempoBased];					
						double dur = [durTime calcTimeWithTempo:tempo quantizer:quantizer isQuantizerTempoBased:isQTempoBased];

						if (dur >= 0) {
							// schedule the instance to stop after the desired duration
							AMValue *val = [[AMValue alloc] initWithValue:(_currentTime + dur)];
							[val autorelease];
							AMTime *time = [[AMTime alloc] initWithTime:val];
							[time autorelease];
							
							[[AMSoundManager getScheduler] registerTimeAction:time playable:instance action:AM_ACT_STOP permanent:NO];

							AM_LOG(@"Cue scheduled to stop.");
						}
					}
				}
				break;
			}
				
			case AM_ACT_STOP:
				AM_LOG(@"Time action stopped.");
				[(id <AMPlayable>)playable stop];
				break;
				
			case AM_ACT_SET:
				// TODO
				//				[(AMPlayable *)playable stop];
				break;
				
			case AM_ACT_ENABLE:
				// TODO
				//				((Chunk)parent).setEnabled(true);
				break;
				
			case AM_ACT_DISABLE:
				// TODO
				//				((Chunk)parent).setEnabled(false);
				break;
				
			case AM_ACT_SET_ACTIVE:
				[(AMSound *)playable setActive];
				break;
			default:
				break;
		}
		
		// If the action was intented to be single-use, dismiss it.
		if (!permanent) {
			begin = -1;
			dismiss = true;
		}
	}
	
	return dismiss;
}


-(void)dealloc 
{
	// release vars
	[playable release];
	
	[super dealloc];
}

@end
