/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  DocumentTableViewController.mm
//  aria
//
//  Created by Yohan on 06/07/10.
//  Copyright 2010 INRIA, Team WAM. All rights reserved.
//

#import "DocumentTableViewController.h"


@interface DocumentTableViewController()
// private methods
- (void)loadDemoDocuments;
- (void)loadOnlineDocuments;
@end

@implementation DocumentTableViewController

@synthesize onlineDocumentsList;


#pragma mark -
#pragma mark View lifecycle

- (void)viewDidLoad {
    [super viewDidLoad];

    // Uncomment the following line to preserve selection between presentations.
    self.clearsSelectionOnViewWillAppear = NO;
 
    // Uncomment the following line to display an Edit button in the navigation bar for this view controller.
	self.navigationItem.rightBarButtonItem = self.editButtonItem;
	
	// Add document button
	addButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemAdd target:self action:@selector(addDocument)];
	self.navigationItem.leftBarButtonItem = addButton;
	
	// init items
	onlineDocumentsList = [[NSMutableArray alloc] init];
	demoDocumentsList = [[NSMutableArray alloc] init];
	
	[self loadDemoDocuments];
	[self loadOnlineDocuments];
}

#pragma mark -
#pragma mark Document loading

- (void)loadDemoDocuments
{
	NSString *path = [[[NSBundle mainBundle] resourcePath] stringByAppendingPathComponent:DEMO_LIST_FILE];	
	NSString *demoDocList = [NSString stringWithContentsOfFile:path encoding:NSUTF8StringEncoding error:nil];
	
    if (demoDocList) {  
		NSArray *elements = [demoDocList componentsSeparatedByString:@"\n"];;
		
		if (elements != nil) {
			Document *currDoc = nil;
			int currLine = 0;
			
			while (currLine < [elements count]) {
				// new document
				currDoc = [[[Document alloc] init] autorelease];
				currDoc.online = NO;
				currDoc.name = [elements objectAtIndex:currLine++];
				if (currLine >= [elements count]) break;
				currDoc.url = [elements objectAtIndex:currLine++];
				if (currLine >= [elements count]) break;
				
				NSMutableString *desc = [NSMutableString stringWithString:[elements objectAtIndex:currLine++]];
				
				// read description
				while (currLine < [elements count] && ![(NSString *)[elements objectAtIndex:currLine] isEqualToString:@";;"]) {
					[desc appendString:@"\n"];					
					[desc appendString:[elements objectAtIndex:currLine++]];
				}
				currDoc.description = [NSString stringWithString:desc];		// store a non-mutable copy of the string
				
				// skip ";;" separator
				currLine++;

//				NSLog(@"Found document!\n-name:'%@'\n-url : '%@'\n-desc: '%@'", currDoc.name, currDoc.url, currDoc.description);
				
				// add document to list
				[demoDocumentsList addObject:currDoc];
			}
		}
    } 
}

- (void)loadOnlineDocuments
{
	NSString *path = [NSSearchPathForDirectoriesInDomains (NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
	path = [path stringByAppendingPathComponent:USER_LIST_FILE];

	NSString *userDocList = [NSString stringWithContentsOfFile:path encoding:NSUTF8StringEncoding error:nil];
	
    if (userDocList) {  
		NSArray *elements = [userDocList componentsSeparatedByString:@"\n"];;
		
		if (elements != nil) {
			Document *currDoc = nil;
			int currLine = 0;
			
			while (currLine < [elements count]) {				
				// new document
				currDoc = [[[Document alloc] init] autorelease];
				currDoc.online = YES;
				currDoc.name = [elements objectAtIndex:currLine++];
				if (currLine >= [elements count]) break;
				currDoc.url = [elements objectAtIndex:currLine++];
				if (currLine >= [elements count]) break;
				
				NSMutableString *desc = [NSMutableString stringWithString:[elements objectAtIndex:currLine++]];
				
				// read description
				while (currLine < [elements count] && ![(NSString *)[elements objectAtIndex:currLine] isEqualToString:@";;"]) {
					[desc appendString:@"\n"];
					[desc appendString:[elements objectAtIndex:currLine++]];
				}
				currDoc.description = [NSString stringWithString:desc];		// store a non-mutable copy of the string
				
				// skip ";;" separator
				currLine++;
				
//				NSLog(@"Found document!\n-name:'%@'\n-url : '%@'\n-desc: '%@'", currDoc.name, currDoc.url, currDoc.description);
				
				// add document to list
				[onlineDocumentsList addObject:currDoc];
			}
		}
    } 
}

#pragma mark -
#pragma mark Document saving

- (void)saveUserDocuments {
	// save online document list
	NSString *userListPath = [NSSearchPathForDirectoriesInDomains (NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
	userListPath = [userListPath stringByAppendingPathComponent:USER_LIST_FILE];
	
	NSMutableString *docList = [NSMutableString string];
	
	for (Document *doc in onlineDocumentsList)
		[docList appendFormat:@"%@\n%@\n%@\n;;\n", doc.name, doc.url, doc.description];
	
	[docList writeToFile:userListPath atomically:YES encoding:NSUTF8StringEncoding error:nil];
}

#pragma mark -
#pragma mark Table view data source

- (void)addDocument
{
	DocumentDetailTableViewController *detailViewController = [[DocumentDetailTableViewController alloc] initWithNibName:@"DocumentDetailTableViewController" bundle:nil];
	
	// set the document to show details of the new document
	Document *doc = [[Document alloc] init];
	doc.online = YES;
	[doc autorelease];
	detailViewController.document = doc;
		
	// activate modal edit mode	
	detailViewController.modalMode = YES;
	
	UINavigationController *nc = [[UINavigationController alloc] initWithRootViewController:detailViewController];
	[self presentModalViewController:nc animated:YES];
	
	[nc release];
	[detailViewController release];
	
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    // return the number of sections.
    return 2;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    // return the number of rows in the section.
	if (section == 0)
		return [onlineDocumentsList count];
	else
		return [demoDocumentsList count];
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section {
	if (section == 0)
		return @"My Documents";
	else
		return @"Examples";
}

// Customize the appearance of table view cells.
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
	static NSString *cellIdentifier = @"DocumentCell";
	
	UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellIdentifier];
	if (cell == nil) {
		cell = [[[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:cellIdentifier] autorelease];
	}
	
    // configure the cell
    Document *doc = nil;
	if (indexPath.section == 0)
		doc = [onlineDocumentsList objectAtIndex:indexPath.row];
	else
		doc = [demoDocumentsList objectAtIndex:indexPath.row];
	
	cell.textLabel.text = doc.name;	
	cell.accessoryType = UITableViewCellAccessoryDetailDisclosureButton;

	return cell;
}

// Override to support conditional editing of the table view.
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {
	if (indexPath.section == 0)
		return YES;
	else
		return NO;
}

// Override to support editing the table view.
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
    
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        // Delete the row from the data source
		if (indexPath.section == 0) {
			// delete folder with cached resources if needed
			Document *doc = [onlineDocumentsList objectAtIndex:indexPath.row];
			NSString *path = (NSString *)[NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
			path = [path stringByAppendingPathComponent:doc.name];
			
			if ([[NSFileManager defaultManager] fileExistsAtPath:path])
				[[NSFileManager defaultManager] removeItemAtPath:path error:NULL];
			
			[onlineDocumentsList removeObjectAtIndex:indexPath.row];
		} else {
			[demoDocumentsList removeObjectAtIndex:indexPath.row];
		}
        [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:UITableViewRowAnimationFade];
    }   
}

// Used to forbids moving the row to undesired places
- (NSIndexPath *)tableView:(UITableView *)tableView targetIndexPathForMoveFromRowAtIndexPath:(NSIndexPath *)sourceIndexPath toProposedIndexPath:(NSIndexPath *)proposedDestinationIndexPath {
	if (sourceIndexPath.section != proposedDestinationIndexPath.section)
		return [NSIndexPath indexPathForRow:([onlineDocumentsList count]-1) inSection:sourceIndexPath.section];
	else 
		return proposedDestinationIndexPath;
}

// Override to support rearranging the table view.
- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath {
	if (fromIndexPath.section == 0) {
		[onlineDocumentsList exchangeObjectAtIndex:fromIndexPath.row withObjectAtIndex:toIndexPath.row];
	} else {
		[demoDocumentsList exchangeObjectAtIndex:fromIndexPath.row withObjectAtIndex:toIndexPath.row];
	}
}

#pragma mark -
#pragma mark Table view delegate

- (void)showLoadingError {
	UIAlertView *alert = [[[UIAlertView alloc] initWithTitle:@"Error" message:@"An error occurred while loading the document! The A2ML document may either contains errors or the download is incomplete." delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil] autorelease];
	[alert show];
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
	[tableView deselectRowAtIndexPath:indexPath animated:YES];
	
	// get document and load it
	Document *doc = nil;
	if (indexPath.section == 0)
		doc = [onlineDocumentsList objectAtIndex:indexPath.row];
	else
		doc = [demoDocumentsList objectAtIndex:indexPath.row];	
	
	if (doc != nil) {
		if ([doc isInCache]) {
			NSError *error = nil;
			[infoViewController updateInfos:[doc load:&error]];
						
			if (error)
				[self showLoadingError];

		} else {
			lastSelectedRow = indexPath.row;
			
			UIAlertView *alert = [[[UIAlertView alloc] init] autorelease];
			[alert setTitle:@"Document missing"];
			[alert setMessage:@"The selected document has not been downloaded yet.\nDo you want to download it now?"];
			[alert setDelegate:self];
			[alert addButtonWithTitle:@"Yes"];
			[alert addButtonWithTitle:@"No"];
			[alert show];	
		}
	}
}

- (void)tableView:(UITableView *)tableView accessoryButtonTappedForRowWithIndexPath:(NSIndexPath *)indexPath {
    // navigation logic may go here. Create and push another view controller.
	DocumentDetailTableViewController *detailViewController = [[DocumentDetailTableViewController alloc] initWithNibName:@"DocumentDetailViewController" bundle:nil];

	// set the document to show details
	if (indexPath.section == 0)
		detailViewController.document = [onlineDocumentsList objectAtIndex:indexPath.row];
	else
		detailViewController.document = [demoDocumentsList objectAtIndex:indexPath.row];	
	
	// pass the selected object to the new view controller.
	[self.navigationController pushViewController:detailViewController animated:YES];
	[detailViewController release];
}

#pragma mark -
#pragma mark Alert view delegate

- (void)alertView:(UIAlertView *)actionSheet clickedButtonAtIndex:(NSInteger)buttonIndex {
	switch (buttonIndex) {
		case 0:		// yes
		{
			DownloadViewController *downloadViewController = [[DownloadViewController alloc] initWithNibName:@"DownloadViewController" bundle:nil];
			downloadViewController.document = [onlineDocumentsList objectAtIndex:lastSelectedRow];
			downloadViewController.reloadSounds = YES;
			[downloadViewController setModalTransitionStyle:UIModalTransitionStyleCrossDissolve];
			[[OverlayViewController sharedOverlayViewController] presentModalViewController:downloadViewController animated:YES];
			[downloadViewController release];
			
			// load doc, hopefully the download has completed well :-)
			NSError *error = nil;
			[infoViewController updateInfos:[(Document *)[onlineDocumentsList objectAtIndex:lastSelectedRow] load:&error]];
			
			if (error != nil)
				[self showLoadingError];
			
			break;
		}
		default:	// no
			return;
			break;
	}
}

#pragma mark -
#pragma mark Memory management

- (void)viewDidDisappear:(BOOL)animated {
	// save user documents
//	[self saveUserDocuments];
	[super viewDidDisappear:animated];
}

- (void)dealloc {
	// release vars
	[onlineDocumentsList release];
	[demoDocumentsList release];
	[addButton release];
	
    [super dealloc];
}

@end

