/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  DownloadViewController.mm
//  aria
//
//  Created by Yohan on 30/07/10.
//  Copyright 2010 INRIA, Team WAM. All rights reserved.
//

#import "DownloadViewController.h"


@implementation DownloadViewController

@synthesize document;
@synthesize reloadSounds;

#pragma mark -
#pragma mark View management

// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
    [super viewDidLoad];
	downloadManager = nil;
}

- (void)viewWillAppear:(BOOL)animated {
	progressLabel.text = @"Initializing download...";
	progressView.progress = 0.0;
}

- (void)viewDidAppear:(BOOL)animated {
	
	// just to be safe
	if (!document)
		[self dismissModalViewControllerAnimated:YES];
	
	// download the A2ML document
	progressLabel.text = @"Downloading A2ML document...";
	progressView.progress = (reloadSounds ? 0.0 : 0.5);
	NSData *docData = [DownloadManager downloadFileAtUrl:document.url];

	NSString *path = (NSString *)[NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
	NSString *basePath = [path stringByAppendingPathComponent:document.name];
	path = [basePath stringByAppendingPathComponent:[document.url lastPathComponent]];

	progressLabel.text = @"Saving A2ML document...";
	progressView.progress = (reloadSounds ? 0.1 : 0.75);
	
	if (docData) {
		[docData writeToFile:path options:NSDataWritingAtomic error:nil];
		
		if (reloadSounds) {
			progressLabel.text = @"Parsing A2ML document...";
			
			NSArray *sndUrls = [document getSoundsURLs];
			
			downloadManager = [[DownloadManager alloc] init];
			[downloadManager setDelegate:self];
			
			if (sndUrls != nil && [sndUrls count] > 0) {				
				progressLabel.text = [NSString stringWithFormat:@"Downloaded sound 0/%d", [sndUrls count]];
				progressView.progress = 0.2;		
				[downloadManager downloadFilesAtUrls:sndUrls toPath:basePath];
			} else {
				progressLabel.text = @"Completed.";
				progressView.progress = 1.0;
				
				[self dismissModalViewControllerAnimated:YES];
			}
			
		} else {
			progressLabel.text = @"Completed.";
			progressView.progress = 1.0;
			
			[self dismissModalViewControllerAnimated:YES];
		}	
	} else {
		UIAlertView *alert = [[[UIAlertView alloc] initWithTitle:@"Error" message:@"Download failed!" delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil] autorelease];
		[alert show];	
	}
}

- (void)dealloc {
	// release vars
	[downloadManager release];
	self.document = nil;

    [super dealloc];
}

#pragma mark -
#pragma mark DownloadManager delegate methods

- (void)didFinishDownload:(int)_index 
{
	progressLabel.text = [NSString stringWithFormat:@"Downloaded sound %d/%d", downloadManager.finishCount, [downloadManager.urls count]];
	progressView.progress = 0.2 + (0.8 * downloadManager.finishCount / [downloadManager.urls count]);		
}

- (void)didFinishAllDownload
{
	progressLabel.text = @"Completed.";
	progressView.progress = 1.0;
	
	[self dismissModalViewControllerAnimated:YES];
}

- (void)didFailDownload:(int)_index withError:(NSError *)_error
{
	UIAlertView *alert = [[[UIAlertView alloc] initWithTitle:@"Error" message:[NSString stringWithFormat:@"Download failed:\n%@", [_error localizedDescription]] delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil] autorelease];
	[alert show];	
}

#pragma mark -
#pragma mark Alert view delegate

- (void)alertView:(UIAlertView *)actionSheet clickedButtonAtIndex:(NSInteger)buttonIndex 
{
	progressLabel.text = @"Download incomplete.";
	progressView.progress = 1.0;
	
	[self dismissModalViewControllerAnimated:YES];
}

@end
