/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  OverlayViewController.mm
//  aria
//
//  Created by Yohan on 30/07/10.
//  Copyright 2010 INRIA, Team WAM. All rights reserved.
//

#import "OverlayViewController.h"


// Shared overlay view controller instance
static OverlayViewController *sharedInstance = nil;

@implementation OverlayViewController

#pragma mark -
#pragma mark Setup view overlay

 - (void)viewDidLoad {
	 [super viewDidLoad];
	 
	 AppDelegate *appDelegate = [[UIApplication sharedApplication] delegate];
	 [appDelegate.window addSubview:self.view];
	 self.view.hidden = YES;
}

#pragma mark -
#pragma mark Singleton methods

+ (OverlayViewController*)sharedOverlayViewController
{
    @synchronized(self)
    {
        if (sharedInstance == nil)
			sharedInstance = [[OverlayViewController alloc] init];
    }
    return sharedInstance;
}

+ (id)allocWithZone:(NSZone *)zone {
    @synchronized(self) {
        if (sharedInstance == nil) {
            sharedInstance = [super allocWithZone:zone];
            return sharedInstance;	// assignment and return on first allocation
        }
    }
    return nil;	// on subsequent allocation attempts return nil
}

- (id)copyWithZone:(NSZone *)zone
{
    return self;
}

- (id)retain {
    return self;
}

- (unsigned)retainCount {
    return UINT_MAX;	// denotes an object that cannot be released
}

- (void)release {
	// do nothing
}

- (id)autorelease {
    return self;
}

@end
