/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  GlassButton.m
//  aria
//
//  Created by Yohan on 29/07/10.
//  Copyright 2010 INRIA, Team WAM. All rights reserved.
//

#import "GlassButton.h"


@implementation GlassButton

#pragma mark -
#pragma mark Glass effect appearance

// Makes the shiny apperance
- (void)setupAppearance {
	self.layer.cornerRadius = self.frame.size.height / 5.5;
	self.layer.masksToBounds = YES;
	self.layer.borderColor = [UIColor colorWithWhite:0.3 alpha:0.75].CGColor;
	self.layer.borderWidth = 1.0;
	
	shineLayer = [CAGradientLayer layer];
    shineLayer.frame = self.layer.bounds;
    shineLayer.colors = [NSArray arrayWithObjects:
						 (id)[UIColor colorWithWhite:1.0 alpha:0.9].CGColor,
						 (id)[UIColor colorWithWhite:1.0 alpha:0.6].CGColor,
						 (id)[UIColor colorWithWhite:1.0 alpha:0.1].CGColor,
						 (id)[UIColor colorWithWhite:0.8 alpha:0.0].CGColor,
						 (id)[UIColor colorWithWhite:0.85 alpha:0.2].CGColor,
						 nil];
    shineLayer.locations = [NSArray arrayWithObjects:
							[NSNumber numberWithFloat:0.0],
							[NSNumber numberWithFloat:0.06],
							[NSNumber numberWithFloat:0.5],
							[NSNumber numberWithFloat:0.5],
							[NSNumber numberWithFloat:1.0],
							nil];
    [self.layer addSublayer:shineLayer];
	
	highlightLayer = [CALayer layer];
    highlightLayer.backgroundColor = [UIColor colorWithWhite:0.0 alpha:0.25].CGColor;
    highlightLayer.frame = self.layer.bounds;
    highlightLayer.hidden = YES;
	highlightLayer.speed = 4.0;
    [self.layer insertSublayer:highlightLayer atIndex:0];
	
	[self setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
	[self setTitleColor:[UIColor colorWithWhite:0.85 alpha:1.0] forState:UIControlStateDisabled];
	
	self.titleLabel.shadowOffset = CGSizeMake(0, -1);
	self.titleLabel.font = [UIFont boldSystemFontOfSize:20];
	self.titleLabel.adjustsFontSizeToFitWidth = YES;
}

#pragma mark -
#pragma mark Initialization

- (id)initWithFrame:(CGRect)aRect {
	if (self = [super initWithFrame:aRect]) {
		[self setupAppearance];		
	}
	return self;
}

- (void)awakeFromNib {
	[super awakeFromNib];
	[self setupAppearance];	
}

- (void)layoutSubviews {
	[super layoutSubviews];
	
	shineLayer.frame = self.layer.bounds;
	highlightLayer.frame = self.layer.bounds;
}

#pragma mark -
#pragma mark Highlight handler

- (void)setHighlighted:(BOOL)highlight {
	[highlightLayer setHidden:!highlight];
    [super setHighlighted:highlight];
}

@end
