/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  StylizedProgressView.m
//  aria
//
//  Created by Yohan on 30/07/10.
//  Copyright 2010 INRIA, Team WAM. All rights reserved.
//

#import "StylizedProgressView.h"


@implementation StylizedProgressView

#pragma mark -
#pragma mark Update progress

- (void)updateProgress {
	CGRect rect = CGRectInset(self.layer.bounds, 2.0, 2.0);
	rect.size.width = floor(rect.size.width * self.progress);
	_progressLayer.frame = rect;
	_shineLayer.frame = _progressLayer.bounds;
}

- (void)setProgress:(float)value {
	[super setProgress:value];
	[self updateProgress];
}

#pragma mark -
#pragma mark Progress bar appearance

// Makes the shiny apperance
- (void)setupAppearance {
	// setup borders
	self.layer.cornerRadius = self.bounds.size.height / 2;
	self.layer.masksToBounds = YES;
	self.layer.borderColor = [UIColor whiteColor].CGColor;
	self.layer.borderWidth = 1.0;
	self.layer.backgroundColor = [UIColor colorWithWhite:0 alpha:0.3].CGColor;
	
	// setup progress bar	
	_progressLayer = [CALayer layer];
	_progressLayer.cornerRadius = self.frame.size.height / 2;
	_progressLayer.masksToBounds = YES;
    _progressLayer.backgroundColor = [UIColor whiteColor].CGColor;
	[self updateProgress];
    [self.layer addSublayer:_progressLayer];
	
	// add some shine to progress indicator
	_shineLayer = [CAGradientLayer layer];
    _shineLayer.frame = _progressLayer.bounds;
    _shineLayer.colors = [NSArray arrayWithObjects:
						  (id)[UIColor colorWithWhite:0.5 alpha:0.1].CGColor,
						  (id)[UIColor colorWithWhite:0.5 alpha:0.5].CGColor,
						  nil];
    _shineLayer.locations = [NSArray arrayWithObjects:
							 [NSNumber numberWithFloat:0.5],
							 [NSNumber numberWithFloat:1.0],
							 nil];
    [_progressLayer addSublayer:_shineLayer];
}

#pragma mark -
#pragma mark Custom drawing

- (void)drawRect:(CGRect)rect {
	// do nothing, only the main layer will be drawn
}

#pragma mark -
#pragma mark Initialization

- (id)initWithFrame:(CGRect)aRect {
	if (self = [super initWithFrame:aRect]) {
		[self setupAppearance];		
	}
	return self;
}

- (void)awakeFromNib {
	[super awakeFromNib];
	[self setupAppearance];	
}

- (void)layoutSubviews {
	[super layoutSubviews];
	[self updateProgress];
}

@end
