/* ***** BEGIN LICENSE BLOCK *****
 *
 * Copyright (c) 2009-2011 Yohan Lasorsa, INRIA <http://wam.inrialpes.fr/>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 * ***** END LICENSE BLOCK ***** */

//
//  Document.h
//  aria
//
//  Created by Yohan on 08/07/10.
//  Copyright 2010 INRIA, Team WAM. All rights reserved.
//

#import "aria.h"
#import <Foundation/Foundation.h>


/** 
 Contains informations on an A2ML document.
 */
@interface Document : NSObject <NSXMLParserDelegate> {
	// Document attributes
	BOOL online;
	NSString *name;
	NSString *url;
	NSString *description;
	
	// Private attributes (for sound URL parsing)
	NSMutableArray *sndUrls;
	NSString *baseURL;
}

/** If the document is local or online. */
@property (nonatomic) BOOL online;

/** The name of the document. */
@property (nonatomic, retain) NSString *name;

/** The URL to the A2ML file. */
@property (nonatomic, retain) NSString *url;

/** The description of the document. */
@property (nonatomic, retain) NSString *description;


/**
 Checks if the document is in cache (only for online documents, return always YES for local documents). 
 @return YES the document is local or in cache, NO if it's an online document that has never been loaded.
 */
- (BOOL)isInCache;

/**
 Loads the A2ML documents and returns informations about it in HTML format
 @param error A pointer to a NSError used to tell if errors occured during loading. Can be NULL.
 @return Informations about the document in HTML format.
 */
- (NSString *)load:(NSError **)_error;

/**
 Gets the URLs of the sounds of an online document, to download them.
 A cached version of the A2ML must exists before calling this method.
 @return An array of NSString objects containing the URLs of the sound of the online document.
 */
- (NSArray *)getSoundsURLs;

@end
